// Cookies.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _COOKIES_H
#define _COOKIES_H

//! Store the cookies name/value pairs.

typedef struct cookiestruct
{
	struct cookiestruct *next; //!< Linked list ptr to next in list 
	char *name; //!< Cookie name
	char *value; //!< Cookie value
} COOKIE;

//! Retrieve and manage cookies during a cgi call.

class Cookies {
public:
	/**
	 * Default constructor (used in POST operations).
	 * Input is read from stdin.
	*/
	Cookies(void);
	/**
	 * Another constructor (used in GET operations).
	 * Input is read from the environment variable QUERY_STRING
	 * with a length of CONTENT_LENGTH.
	 * @param query_string The httpd server provided QUERY_STRING
	*/
	Cookies(char *query_string);
	/**
	 * Destructor.
	*/
	~Cookies(void);
	int getvalue(char *,char *,short); // (name, buffer, length)
	void replacevalue(char *,char *);
	void replacevalue(char *,long);
	void replacevalue(char *,int);
	int getlength(char *);
	void setcookie(char *,char *,char *,char *);
	void setcookie(char *,char *,char *,long);
	void setcookie(char *,char *,char *,int);
	void expiredatetime(char *);

private:
	COOKIE *cookiebase;
};

#endif // _COOKIES_H
/**
 **	Form.h - read stdin, parse cgi input
 **
 **	Written: 1999-Feb-10 grymse@alhem.net
 **/

/*
Copyright (C) 1999  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _FORM_H
#define _FORM_H

#include <string>

/**
 * Store the name/value pairs from a GET/POST operation.
 * "name" does not have to be unique.
*/

typedef struct cgistruct
{
	struct cgistruct *next; //!< Linked list ptr to next object
	char *name; //!< Name of parameter
	char *value; //!< Value of parameter
	char *path; //!< Path to file if parameter was a posted file
} CGI;

//! Process and store user input from a GET/POST operation.

class Form
{
public:
	/**
	 * Default constructor (used in POST operations).
	 * Input is read from stdin. Number of characters to read
	 * can be found in the environment variable CONTENT_LENGTH.
	*/
	Form(void);
	/**
	 * Another constructor (used in GET operations).
	 * Input is read from the environment variable QUERY_STRING.
	 * @param query_string The httpd server provided QUERY_STRING
	 * @param length Query string length.
	*/
	Form(char *query_string,int length);
	~Form(void);
	void EnableRaw(bool);

	void strcpyval(char *,char *,size_t);

	/* get names */
	bool getfirst(char *,short);
	bool getnext(char *,short);

	/* get names and values */
	bool getfirst(char *,short,char *,short);
	bool getnext(char *,short,char *,short);

	/* get value */
	int getvalue(char *,char *,short);
	std::string getvalue(const std::string& );
	size_t getlength(char *);
	CGI *getbase(void) { return cgibase; }

	char *GetBoundary();

private:
	CGI *cgibase;
	CGI *current;
	char *m_strBoundary;
	bool raw;
};
#endif // _FORM_H
/**
 **	Parse.h - parse a string
 **
 **	Written: 1999-Feb-10 grymse@alhem.net
 **/

/*
Copyright (C) 1999  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _PARSE_H
#define _PARSE_H

/***************************************************/
/* interface of class Parse */

//! Splits a string whatever way you want

class Parse
{
public:
	Parse();
	Parse(char *);
	Parse(char *,char *);
	Parse(char *,char *,short);
	~Parse();
	short issplit(char);
	void getsplit(void);
	void getsplit(char *);
	void getword(void);
	void getword(char *);
	void getword(char *,int);
	void getword(char *,char *,int);
	void getrest(char *);
	long getvalue(void);
	void setbreak(char);
	int getwordlen(void);
	int getrestlen(void);
	void enablebreak(char c) {
		pa_enable = c;
	}
	void disablebreak(char c) {
		pa_disable = c;
	}
	void getline(void);
	void getline(char *);
	void getline(char *,int);
	int getptr(void) { return pa_the_ptr; }
	void EnableQuote(bool b) { pa_quote = b; }

private:
	char *pa_the_str;
	char *pa_splits;
	char *pa_ord;
	int   pa_the_ptr;
	char  pa_breakchar;
	char  pa_enable;
	char  pa_disable;
	short pa_nospace;
	bool  pa_quote;
};

#endif // _PARSE_H
/**
 **	list.h - linked list stuff
 **
 **	Written: 1999-Feb-10 grymse@alhem.net
 **/

/*
Copyright (C) 1999  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _LIST_H
#define _LIST_H


//! Base linked list struct

typedef struct liststruct
{
	struct liststruct *next;
} LIST;

//! Base linked string list struct

typedef struct sliststruct
{
	struct sliststruct *next;
	char s[0];
} SLIST;

void addlist(void *_base,void *item);
void addlistl(void *_base,void *item);
void removelist(void *_base,void *item);
void savelist(void *_base,int size,char *filename);
void loadlist(void *_base,int size,char *filename);
void addlistsort(void *_base,SLIST *item);
void freelist(void *);
long listlen(void *_base);

#ifdef WIN32
#define strcasecmp stricmp
#endif

#endif // _LIST_H
// Header.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _HEADER_H
#define _HEADER_H

typedef struct headerstruct
{
	struct headerstruct *next;
	char *name;
	char *value;
} HEADER;

class Header {
public:
	Header(void);
	Header(char *,int);
	~Header(void);
	int getvalue(char *,char *,short); // (name, buffer, length)
	void replacevalue(char *,char *);
	int getlength(char *);

	/* get names */
	void getfirst(char *,short);
	void getnext(char *,short);
	int getindex(void) { return dataindex; }

private:
	HEADER *headerbase;
	HEADER *current;
	int dataindex;
};

#endif // _HEADER_H
// Readhtml.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _READHTML_H
#define _READHTML_H

#define MAXL 20000

class Readhtml {
public:
	Readhtml(void);
	Readhtml(char *);
	~Readhtml(void);
	int getnext(char *,int);

private:
	char *fn;
	FILE *fil;
	char peekc;
	char *buf;
};

#endif // _READHTML_H
// WebForm.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _WEBFORM_H
#define _WEBFORM_H

#include <string>

class Web;


//! The CGI forms class

class WebForm
{
public:
  WebForm(Web *,const std::string &);	// Parent, Form unique name
  virtual ~WebForm();

  virtual void Process() = 0;		// FormInput

  virtual void Select(const std::string &) = 0; // <SELECT name=>...</SELECT>
  virtual void List() = 0;		// List
  virtual void Display( long = 0 ) = 0;	// Form

  Form *GetForm();
  Cookies *GetCookies();
  std::string GetCgiName();
  Web *GetWeb();
  
  std::string GetDate();
  std::string GetFormName();
  virtual void nbprintf(char *format, ...);
  virtual void eprintf(char *format, ...);

protected:
  Web *m_pclWeb;
  std::string m_strFormName;

private:
};


#endif // _WEBFORM_H
// Web.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _WEB_H
#define _WEB_H

#include <string>
#include <vector>

class Form;
class Cookies;

//! The CGI main class

class Web 
{
public:
	Web();
	Web(const std::string &,const std::string &);
	Web(int argc,char *argv[],const std::string &,const std::string &);
	virtual ~Web();

	void Init();

	void SetCookieDomain(const std::string &);
	void SetCookiePath(const std::string &);
	void SetCookie(const std::string &name,const std::string &value);
	void SetCookie(const std::string &name,long value);
	void SetCookie(const std::string &name,int value);
	void Execute();
	void SetCgiName(const std::string &);
	std::string GetCgiName();
	Form *GetForm();
	Cookies *GetCookies();

	void RegWebForm(WebForm *);

	virtual void ReadCookies();
	virtual void FormInput();
	virtual void CreateHeader();
	virtual void GenerateDocument();

	std::string GetDate();
	std::string GetRequestMethod();
	void UpdateModified(const std::string& );
	std::string datetime2httpdate(const std::string& dt);
	const std::string& GetLastModified() { return m_lastmodified; }

protected:
	Form *m_pclForm;
	Cookies *m_pclCookies;

private:
	std::string m_strCookieDomain;
	std::string m_strCookiePath;
	std::string m_strCgiName;
	std::vector<WebForm *> m_forms;
	std::string m_date;
	std::string m_method; // request_method
	std::string m_lastmodified;
};


#endif // _WEB_H
// MenuWeb.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _MENUWEB_H
#define _MENUWEB_H


//! Menuitems

enum
{
	MENUITEM_TYPE_FORM = 1,
	MENUITEM_TYPE_SEPARATOR,
	MENUITEM_TYPE_URL,
	MENUITEM_TYPE_IMAGE
};


//! Menu styles enumerated

enum 
{
	MENU_STYLE_FRAME_LEFT = 1,
	MENU_STYLE_TOP,
	MENU_STYLE_LEFT
};


//! Menu item definition struct

typedef struct
{
	int num;
	int type;
	WebForm *form;
	std::string title;
	std::string url; // <a href=...>
	std::string src; // <img src=...>
} MENUITEM;
typedef std::vector<MENUITEM *> menuitem_v;


//! Automatic menu creation functionality added to Web class

class MenuWeb : public Web
{
public:
	MenuWeb(int = MENU_STYLE_TOP);
	MenuWeb(const std::string &,const std::string &,int = MENU_STYLE_TOP);
	MenuWeb(int argc,char *argv[],const std::string &,const std::string &,int = MENU_STYLE_TOP);
	~MenuWeb();

	void SetMenuStyle(int);

	const MENUITEM & AddMenuItem(WebForm *,const std::string &title);
	void AddMenuSeparator();
	void AddMenuLink(const std::string &url,const std::string &title);
	void AddMenuImage(const std::string &src);

	virtual void ReadCookies();
	virtual void FormInput();
	virtual void CreateHeader();
	virtual void GenerateDocument();

	std::string GetCookiename() { return m_cookiename; }
	void SetCookiename(const std::string &str) { m_cookiename = str; }
	void SetPage(int x) { m_page = x; m_bSetCookie = true; }
	int GetPage() { return m_page; }
	void SetMenuclass(const std::string &x) { m_strClass = x; }

	void SetTheCookie();

private:
	int m_count;
	int m_style;
	int m_page;
	std::string m_cookiename;
	bool m_bSetCookie;
	menuitem_v m_items;
	std::string m_strClass;
};


#endif // _MENUWEB_H
// Table_i.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _TABLE_I_H
#define _TABLE_I_H

//! Create a nice looking html table

class Table
{
public:
	Table(const std::string &color, int padding = 5, const char *caption = NULL,
		const char *textcolor = NULL, FILE *fil = stdout) : m_fil(fil)
	{
		fprintf(m_fil,"<table cellpadding=0 cellspacing=0><tr><td bgcolor=\"%s\">",color.c_str());
		if (caption)
		{
			fprintf(m_fil,"<font color=\"%s\">%s</font></td></tr>",
				textcolor ? textcolor : "#ffffff", caption);
			fprintf(m_fil,"<tr><td bgcolor=\"%s\">",color.c_str());
		}
		fprintf(m_fil,"<table width=\"100%%\" cellspacing=1 cellpadding=%d>",padding);
		fflush(m_fil);
	}
	~Table() {
		fflush(m_fil);
		fprintf(m_fil,"</table>");
		fprintf(m_fil,"</td></tr></table>");
		fflush(m_fil);
	}

private:
	FILE *m_fil;
};


class Table100
{
public:
	Table100(const std::string &color, int padding = 5, const char *caption = NULL,
		const char *textcolor = NULL, FILE *fil = stdout) : m_fil(fil)
	{
		fprintf(m_fil,"<table width=\"100%%\" cellpadding=0 cellspacing=0><tr><td bgcolor=\"%s\">",color.c_str());
		if (caption)
		{
			fprintf(m_fil,"<font color=\"%s\">%s</font></td></tr>",
				textcolor ? textcolor : "#ffffff", caption);
			fprintf(m_fil,"<tr><td bgcolor=\"%s\">",color.c_str());
		}
		fprintf(m_fil,"<table width=\"100%%\" cellspacing=1 cellpadding=%d>",padding);
		fflush(m_fil);
	}
	~Table100() {
		fflush(m_fil);
		fprintf(m_fil,"</table>");
		fprintf(m_fil,"</td></tr></table>");
		fflush(m_fil);
	}

private:
	FILE *m_fil;
};


#endif // _TABLE_I_H
// Ticks_i.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _TICKS_I_H
#define _TICKS_I_H

#include <sys/time.h>
#include <unistd.h>

class Ticks
{
public:
	void Start() {
		struct timeval tv;
		struct timezone tz;
		gettimeofday(&tv,&tz);
		d0 = tv.tv_sec;
		d0 *= 1000000;
		d0 += tv.tv_usec;
		dstop = d0;
		dtemp = d0;
	}
	Ticks() {
		Start();
	}
	~Ticks() {
	}
	void Stop() {
		struct timeval tv;
		struct timezone tz;
		gettimeofday(&tv,&tz);
		dstop = tv.tv_sec;
		dstop *= 1000000;
		dstop += tv.tv_usec;
	}
	double Time(void) {
		return dstop - d0;
	}
	void Time(char *s) {
		sprintf(s,"%.4f sec",Time() / 1000000);
	}
	void TempTime(char *s) {
		struct timeval tv;
		struct timezone tz;
		double dnow;
		gettimeofday(&tv,&tz);
		dnow = tv.tv_sec;
		dnow *= 1000000;
		dnow += tv.tv_usec;
		double d = dnow - dtemp;
		dtemp = dnow;
		sprintf(s,"%.4f sec",d / 1000000);
	}
private:
	double d0;
	double dstop;
	double dtemp;
};


#endif // _TICKS_I_H
// Base64.h
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _BASE64_H
#define _BASE64_H

#include <string>
#include <vector>

class Base64
{
public:
	Base64();
	~Base64();

	// file to file
	void encode_file(char *s,char *outfn);

	// input buffer to std::vector<std::string>
	void encode64(char *in,size_t len,std::vector<std::string> &out);

	// file to std::vector<std::string>
	void encode_from_file(char *,std::vector<std::string> &);

	// file to file
	void decode_file(char *s,char *outfn);

	// std::vector<std::string> to file
	void decode_to_file(std::vector<std::string> &body,char *outfn);

	// file to FILE *
	void decode_stdout(char *fn,FILE *out = stdout);

	// char * to FILE *
	void decode_to_stdout(const std::string &,FILE * = stdout);

	// std::vector<std::string> to FILE *
	void decode_to_stdout(std::vector<std::string> &,FILE * = stdout);

private:
	void setupbase64();
	int setupdecode(char c);
	int reverse(char c);
	void encode(char *in,char *out);
	int decode(char c);
	void decode_string(char *s, unsigned char *output, int &l);
};


#endif // _BASE64_H
